# -*-python-*-

__package__    = "growler.py"
__version__    = "1.0"
__author__     = "Aaron Straup Cope"
__url__        = "http://www.aaronland.info/python/growler/"
__cvsversion__ = "$Revision: 1.4 $"
__date__       = "$Date: 2007/12/29 07:26:25 $"
__copyright__  = "Copyright (c) 2007 Aaron Straup Cope. Perl Artistic License."

import string
import md5
import urllib
import os
import os.path
import Growl
import tempfile
import time
import codecs
import utils

class growler :

    def __init__ (self, cfg) :

        self.cfg = cfg
        
        self.nfc = None
        self.growl = None        
        self.store = None

        self.thumbnames = {}
        self.last_thumb_purge = 0
        
    #

    def setup (self) :

        if not self.setup_growl() :
            return False

        if not self.setup_store() :
            return False

        return True

    #

    def setup_growl (self) :

        try : 
            self.growl = Growl.GrowlNotifier(applicationName=self.name, notifications=[self.name], defaultNotifications=[self.name])
            self.growl.register()
        except Exception, e :
            self.log("failed to set up growl : %s" % e)            
            return False

        return True

    #
    
    def setup_store (self) :

        store = 'default'

        try :
            store = self.cfg.get(self.name, 'datastore')
        except :
            pass

        try :
            if store == 's3' :

                key = self.cfg.get(self.name, 's3_access_key')
                secret = self.cfg.get(self.name, 's3_secret_key')
                prefix = self.cfg.get(self.name, 's3_bucket_prefix')                

                from datastore.aws import s3
                self.store = s3(key, secret, prefix)
                self.store.open()
                
            else :

                path = self.cfg.get(self.name, "storepath")                
                
                from datastore.fs import dbm
                self.store = dbm(path)            
                self.store.open()
                self.store.close()
                
        except Exception, e :
            self.log("failed to set up %s store : %s" % (store, e))
            return False
            
        return True
            
    #

    def loop (self) :

        if not self.setup() :
            return False

        self.run()

    #
    
    def run (self) :

        if self.ensure_nearbyness() :

            try : 
                self.store.open()
            except Exception, e:
                self.log("can not open datastore, skipping: %s" % e)
                self.sleep()
                return
            
            events = self.fetch_events()
            
            if len(events) :
                for event in events :
                    self.notify(event)
                    
            self.store.close()
                
        self.purge_thumbdir()
        self.sleep()

    #

    def ensure_nearbyness(self) :

        try :
            import nfc
        except Exception, e :
            return True

        if not self.nfc :

            try :
                import nfc
                self.nfc = nfc.probe('bluetooth')
            except Exception, e :
                self.log("failed to load nfc libraries : %s" % e)
                return False

        #

        self.log("ensuring nearbyness")
        
        if self.has_stoplist_devices() :
            self.log("spotted a stoplist device, nearbyness fails")
            return False
        
        if not self.has_watchlist_devices() :
            self.log("unable to locate any watchlist devices, nearbyness fails")            
            return False

        return True

    #

    def has_watchlist_devices (self) :

        try : 
            required = self.cfg.get(self.name, "nfc_beacon_watchlist")
        except Exception, e :
            return True

        return self._has_devices(required)

    #
    
    def has_stoplist_devices (self) :
        
        try : 
            other = self.cfg.get(self.name, "nfc_beacon_stoplist")
        except Exception, e :
            return False

        return self._has_devices(other)

    #
    
    def _has_devices (self, devices) :

        devices = map(string.lower, devices.split(","))
        addr = self.nfc.is_any_device_nearby(devices)

        if addr :
            self.log("%s is nearby" % addr)
            return True

        return False
        
    #

    def strip_markup (self, txt) :
        markup = utils.markup()
        return markup.strip(txt)
    
    #
    
    def sleep (self) :
        secs = self.cfg.get(self.name, "interval")
        self.log("sleep for %s seconds" % secs)
        
        time.sleep(float(secs))
        self.run()
        
    #
    
    def fetch_events (self) :
        return self.parse_events("")
    
    #
    
    def parse_events (self, xml) :
        return []
    
    #
    
    def fetch_thumb (self, url) :

        thumb = self.get_thumbname(url)

        if not thumb :
            return None
        
        if not os.path.exists(thumb) :
           urllib.urlretrieve(url, thumb)
           
        return Growl.Image.imageFromPath(thumb)

    #

    def get_thumbdir (self) :

        try : 
            thumbdir = self.cfg.get(self.name, "thumb_folder")
            
            if not os.path.exists(thumbdir) : 

                fs = utils.filesystem()

                try :
                    fs.mkdir(thumbdir)
                except Exception, e :
                    self.log("failed to create %s, %s" % (thumbdir, e))
                    return None

                self.log("created directory %s" % thumdir)
                
            return thumbdir
            
        except Exception, e :
            return tempfile.gettempdir()

    #

    def get_thumbname (self, url) :

        if not self.thumbnames.has_key(url) :
            root = self.get_thumbdir()
            fname = "%s-%s.jpg" % (self.name, md5.md5(url).hexdigest())
            self.thumbnames[url] = os.path.join(root, fname)
            
        return self.thumbnames[url]
    
    #

    def is_custom_thumbdir (self) :

        try :
            self.cfg.get(self.name, "thumb_folder")
            return True
        except Exception, e :
            return False
        
    #
    
    def purge_thumbdir (self) :

        if not self.is_custom_thumbdir() :
            return False

        try :
            self.cfg.get(self.name, "thumb_max_age")
        except Exception, e :
            self.log("no max age defined for purging thumbnails, skipping")
            return False

        timestring = self.cfg.get(self.name, "thumb_max_age")
        timeutil = utils.time()

        offset = timeutil.parse_timestring(timestring)

        if not offset :
            self.log("failed to parse timestring %s" % timestring)
            return False

        thumbdir = self.get_thumbdir()

        if not thumbdir :
            return False
        
        for fname in os.listdir(thumbdir) :

            if not fname.startswith(self.name) :
                self.log("skip starts %s" % fname)
                continue

            if not fname.endswith(".jpg") :
                self.log("skip ends %s" % fname)                
                continue
            
            path = os.path.join(thumbdir, fname)
            mtime = os.stat(path)[9]
            
            now = int(time.time())

            if mtime <= now - offset :

                try :
                    os.unlink(path)
                    self.log("removed old thumb %s" % path)
                except Exception, e :
                    self.log("failed to remove old thumb %s, %s" % (path, e))
                
        return True
    
    #
    
    def format_event (self, event) :
        return "please subclass me"

    #

    def log (self, msg) :
        ts = time.ctime(time.time())
        print "[%s][%s] %s" % (ts, self.name, msg)
